/* firesokker-players.js
 *
 * Allow player sorting.
 *
 * @author Christian Graffer
 */

/* Function: fsok_makePlayersSortable
 *
 * This function gets the player-data and adds information to the
 * tr elements that is needed for sorting.
 *
 * @param obj   doc         the document node
 *
 * @return void
 */
var player_divs_to_sort = new Array();
var clear_divs = new Array();
var fsok_players_document = '';

function fsok_makePlayersSortable(doc) {
  if (fsok_prefs.getBoolPref("player_sorting")) {

	// get all the tables and modify the specified ones
	var divs = doc.getElementsByTagName('div');
	var isown = true;
	var statistics = new Object;
	statistics.strangers = 0;
	statistics.injuries = 0;
	statistics.injurydays = 0;
	statistics.yellow_cards_1 = 0;
	statistics.yellow_cards_2 = 0;
	statistics.red_cards = 0;
	statistics.currency_symbol = '';
	statistics.injury_separator = ' = ';
	fsok_players_document = doc;
	player_divs_to_sort = new Array();
	clear_divs = new Array();
	var players = new Array();

	for (var i = 0; i < divs.length; i++) {
	  if (divs[i].className == 'space2Bottom') {
		var player = fsok_parsePlayerData(divs[i]);
		players[players.length] = player;
		if (!player.shows_all_skills)
		  isown = false;
		for (attribute in player) {
		  if (attribute != 'shows_all_skills')
			divs[i].setAttribute(attribute,player[attribute]);
		}
		player_divs_to_sort[player_divs_to_sort.length] = divs[i];
		divs[i].parentNode.setAttribute('id','sortdiv');
	  } else if (divs[i].style.clear == 'left') {
		clear_divs[clear_divs.length] = divs[i];
	  }
	}
	fsok_addSortSelect(doc,isown);
	fsok_addSummary(doc);
  }

  /* Function: fsok_parsePlayerData
   *
   * This checks whether the user has a plus subscription or not.
   *
   * @param   obj   player_table   the table node of the player
   *
   * @return  obj
   */
  function fsok_parsePlayerData(player_table) {
	var player_data = player_table.getElementsByTagName('div')[0];
	var player_data2 = player_table.getElementsByTagName('div')[1];
	var player_text = fsok_getTextOnly(player_data);
	var player_text2 = fsok_getTextOnly(player_data2);
	var player = new Object;
	var name = player_data.getElementsByTagName('a')[0].innerHTML.split(' ');
	player.shows_all_skills = true;
	player.firstname = name.shift();
	player.surname = name.join(" ");
	player.age = player_text.split(',')[1].trim().replace(/.*\s/g,'');
	player.value = player_data2.innerHTML.split('<br>')[0].replace(/.*\<span/g,'').replace(/\D/g,'');
	player.salary = player_data2.innerHTML.split('<br>')[1].replace(/.*\<span/g,'').replace(/\D/g,'');
	player.cards = 0;
	player.isinjured = false;

	// check how many images with yellow or red cards we find
	var imgs = player_table.getElementsByTagName('img');
	for (var i = 0; i < imgs.length; i++) {
	  if (imgs[i].src == img_ycard) {
		player.cards += 1;
	  } else {
		if (imgs[i].src == img_rcard) {
		  statistics.red_cards += 1;
		  player.cards += 3;
		} else {
		  if (imgs[i].src.substr(0,34) == flag_img_start_string) {
			statistics.strangers += 1;
		  } else {
			if (imgs[i].src == img_injury) {
			  statistics.injuries += 1;
			  player.isinjured = true;
			}
		  }
		}
	  }
	}
	if (player.cards == 2) {
	  statistics.yellow_cards_2 += 1;
	} else if (player.cards == 1) {
	  statistics.yellow_cards_1 += 1;
	}


	if (statistics.currency_symbol == '') {
	  statistics.currency_symbol = player_data2.innerHTML.split('<br>')[0].trim().replace(/.*\d+\s/g,'');
	}

	// here comes the tough part... checking all the skills
	player.form = '00';
	var player_form = player_text2.split("\n")[3].trim().split(' ');
	if (fsok_current_language == 'br' || fsok_current_language == 'mk' || fsok_current_language == 'hr') {
	  player_form = player_form.slice(0,player_form.length - 2).join(' ');
	} else {
	  player_form = player_form.slice(0,player_form.length - 1).join(' ');
	}

	var search1 = /(\(.*\))/g;
	var search2 = /(\d+)/;
	player.injuries = '00';
	if (player.isinjured && player_text2.search(search1) > 0) {
	  var searchresult = player_text2.match(search1);
	  var temp = searchresult[searchresult.length - 1];
	  if (temp.search(search2) > 0) {
		temp.match(search2);
		player.injuries = (parseInt(RegExp.$1) > 9) ? RegExp.$1 : '0'+RegExp.$1;
		statistics.injurydays += parseInt(RegExp.$1);
	  } else {
		player.injuries = '07';
		statistics.injurydays += 7;
		statistics.injury_separator = ' &lt;= ';
	  }
	}

	if (fsok_language.skills[player_form] != null) {
	  player.form = (fsok_language.skills[player_form] > 9) ? fsok_language.skills[player_form] : '0'+fsok_language.skills[player_form];
	} else {
	  fsok_log(player_form,document);
	}

	var tds = player_table.getElementsByTagName('td');
	// check whether all the skills are visible or not
	if (tds.length < 8) {
	  player.shows_all_skills = false;
	} else {
	  player.stamina = '00';
	  player.pace = '00';
	  player.technique = '00';
	  player.passing = '00';
	  player.keeper = '00';
	  player.defender = '00';
	  player.playmaker = '00';
	  player.scorer = '00';
	  var stamina = tds[0].getElementsByTagName('strong')[0].innerHTML.replace(/\<\/.*/g,'').replace(/.*\>/g,'');
	  var pace = tds[2].getElementsByTagName('strong')[0].innerHTML.replace(/\<\/.*/g,'').replace(/.*\>/g,'');
	  var technique = tds[4].getElementsByTagName('strong')[0].innerHTML.replace(/\<\/.*/g,'').replace(/.*\>/g,'');
	  var passing = tds[6].getElementsByTagName('strong')[0].innerHTML.replace(/\<\/.*/g,'').replace(/.*\>/g,'');
	  var keeper = tds[1].getElementsByTagName('strong')[0].innerHTML.replace(/\<\/.*/g,'').replace(/.*\>/g,'');
	  var defender = tds[3].getElementsByTagName('strong')[0].innerHTML.replace(/\<\/.*/g,'').replace(/.*\>/g,'');
	  var playmaker = tds[5].getElementsByTagName('strong')[0].innerHTML.replace(/\<\/.*/g,'').replace(/.*\>/g,'');
	  var scorer = tds[7].getElementsByTagName('strong')[0].innerHTML.replace(/\<\/.*/g,'').replace(/.*\>/g,'');
	  if (fsok_language.skills[stamina]) {
		player.stamina = (fsok_language.skills[stamina] > 9) ? fsok_language.skills[stamina] : '0'+fsok_language.skills[stamina];
	  }
	  if (fsok_language.skills[pace]) {
		player.pace = (fsok_language.skills[pace] > 9) ? fsok_language.skills[pace] : '0'+fsok_language.skills[pace];
	  }
	  if (fsok_language.skills[technique]) {
		player.technique = (fsok_language.skills[technique] > 9) ? fsok_language.skills[technique] : '0'+fsok_language.skills[technique];
	  }
	  if (fsok_language.skills[passing]) {
		player.passing = (fsok_language.skills[passing] > 9) ? fsok_language.skills[passing] : '0'+fsok_language.skills[passing];
	  }
	  if (fsok_language.skills[keeper]) {
		player.keeper = (fsok_language.skills[keeper] > 9) ? fsok_language.skills[keeper] : '0'+fsok_language.skills[keeper];
	  }
	  if (fsok_language.skills[defender]) {
		player.defender = (fsok_language.skills[defender] > 9) ? fsok_language.skills[defender] : '0'+fsok_language.skills[defender];
	  }
	  if (fsok_language.skills[playmaker]) {
		player.playmaker = (fsok_language.skills[playmaker] > 9) ? fsok_language.skills[playmaker] : '0'+fsok_language.skills[playmaker];
	  }
	  if (fsok_language.skills[scorer]) {
		player.scorer = (fsok_language.skills[scorer] > 9) ? fsok_language.skills[scorer] : '0'+fsok_language.skills[scorer];
	  }
	}
	return player;
  }

  /* Function: fsok_addSortSelect
   *
   * Add a select box to the page
   *
   * @param   obj   doc   the document node
   * @param   bool  isown true if all players' skills are visible
   *
   * @return  void
   */
  function fsok_addSortSelect(doc,isown) {
	var div;
	if (fsok_prefs.getBoolPref("show_team_links"))
	  div = doc.getElementsByTagName('div')[21];
	else
	  div = doc.getElementsByTagName('div')[20];

	div.style.display = 'inline';
	div.style.width = '100%';
	var select =  fsok_language.sort_by+' '+
	  '<select style="display: inline; float: none;" name="sort_type" id="fsok_sort_type">'+
	  '<option value="surname">'+fsok_language.surname+'</option>'+
	  '<option value="firstname">'+fsok_language.firstname+'</option>'+
	  '<option value="age">'+fsok_language.age+'</option>'+
	  '<option value="value">'+fsok_language.value+'</option>'+
	  '<option value="salary">'+fsok_language.salary+'</option>'+
	  '<option value="cards">'+fsok_language.cards+'</option>'+
	  '<option value="injuries">'+fsok_language.injuries+'</option>'+
	  '<option '+(isown ? 'style="border-bottom: 1px solid lightgrey;"': '')+' value="form">'+fsok_language.skillnames.form+'</option>';

	if (isown) {
	  select += '<option value="stamina">'+fsok_language.skillnames.stamina+'</option>'+
		'<option value="pace">'+fsok_language.skillnames.pace+'</option>'+
		'<option value="technique">'+fsok_language.skillnames.technique+'</option>'+
		'<option value="passing">'+fsok_language.skillnames.passing+'</option>'+
		'<option value="keeper">'+fsok_language.skillnames.keeper+'</option>'+
		'<option value="defender">'+fsok_language.skillnames.defender+'</option>'+
		'<option value="playmaker">'+fsok_language.skillnames.playmaker+'</option>'+
		'<option value="scorer">'+fsok_language.skillnames.scorer+'</option>';
	}

	select += '</select>&nbsp;<a id="fsok_asc_arrow" href="#"><img src="chrome://firesokker/content/icons/up_arrow.png" style="display:inline;" border="0" /></a>';
	div.innerHTML = div.innerHTML + '<div class="content" style="margin-left: 200px; display: inline;">' + select + '</div>';
	doc.getElementById('fsok_sort_type').addEventListener('change',fsok_changeSortingOnPage,false);
	doc.getElementById('fsok_asc_arrow').addEventListener('click',fsok_changeSortingOnPage,false);
  }

  /* Function: fsok_addSummary
   *
   * Add an overview over all players at the top of the page
   *
   * @param   obj   doc   the document node
   *
   * @return  void
   */
  function fsok_addSummary(doc) {
	var age = 0;
	var avg_age = 0;
	var value = 0;
	var avg_value = 0;
	var salary = 0;
	var avg_salary = 0;
	var form = 0;
	var avg_form = 0;

	for (var i = 0; i < players.length; i++) {
	  age += parseInt(players[i].age);
	  value += parseInt(players[i].value);
	  salary += parseInt(players[i].salary);
	  if (players[i].form == "08")
		form += 8;
	  else if (players[i].form == "09")
		form += 9;
	  else
		form += parseInt(players[i].form);
	  //fsok_log(i + ". " + players[i].form + " / " + form + ": "+players[i].surname+", "+players[i].firstname,doc);
	}

	avg_value = value / players.length;
	avg_age = age / players.length;
	avg_salary = salary / players.length;
	avg_form = form / players.length;
	var tab = doc.createElement('div');
	tab.className = 'container';

	var new_div_1 = doc.createElement('div');
	new_div_1.className = 'topLeft';
	new_div_1.innerHTML = '<div class="topRight"></div>';

	var new_div_2 = doc.createElement('div');
	new_div_2.className = 'leftLine';
	var new_div_2_1 = doc.createElement('div');
	new_div_2_1.className = 'rightLine';
	new_div_2.appendChild(new_div_2_1);
	var new_div_2_1_a = doc.createElement('div');
	new_div_2_1_a.className = 'header';
	new_div_2_1_a.innerHTML = '<div class="titleBlock1">'+fsok_language.summary+':</div>';
	new_div_2_1.appendChild(new_div_2_1_a);
	var new_div_2_1_b = doc.createElement('div');
	new_div_2_1_b.className = 'insideContainer space2Top';
	new_div_2_1.appendChild(new_div_2_1_b);
	var new_div_2_1_b_1 = doc.createElement('div');
	new_div_2_1_b_1.className = 'insideTopLeft';
	new_div_2_1_b_1.innerHTML = '<div class="insideTopRight"></div>';
	var new_div_2_1_b_2 = doc.createElement('div');
	new_div_2_1_b_2.className = 'content';
	new_div_2_1_b.appendChild(new_div_2_1_b_1);
	new_div_2_1_b.appendChild(new_div_2_1_b_2);

	var new_div_3 = doc.createElement('div');
	new_div_3.className = 'bottomLeft';
	new_div_3.innerHTML = '<div class="bottomRight"></div>';




	var tab_content = '<table style="width: 100%;"><tbody>'+
	  '<tr><td style="width: 28%;"><b>'+fsok_language.avg_value+':</b></td>'+
	  '<td style="text-align: right; width=18%">'+formatNumber(avg_value,0,' ','','',' '+statistics.currency_symbol,'','')+'</td><td style="width: 8%">&nbsp;</td>'+
	  '<td style="width: 28%;"><b>'+fsok_language.tot_value+':</b></td>'+
	  '<td style="text-align: right; width=18%">'+formatNumber(value,0,' ','','',' '+statistics.currency_symbol,'','')+'</td></tr>'+
	  '<tr><td colspan="5"><div class="separatorLine" style="width: 100%"></div></td></tr>'+
	  '<tr><td style="width: 28%;"><b>'+fsok_language.avg_salary+':</b></td>'+
	  '<td style="text-align: right; width=18%">'+formatNumber(avg_salary,0,' ','','',' '+statistics.currency_symbol,'','')+'</td><td style="width: 8%">&nbsp;</td>'+
	  '<td style="width: 28%;"><b>'+fsok_language.tot_salary+':</b></td>'+
	  '<td style="text-align: right; width=18%">'+formatNumber(salary,0,' ','','',' '+statistics.currency_symbol,'','')+'</td></tr>'+
	  '<tr><td colspan="5"><div class="separatorLine" style="width: 100%"></div></td></tr>'+
	  '<tr><td style="width: 28%;"><b>'+fsok_language.avg_age+':</b></td>'+
	  '<td style="text-align: right; width=18%">'+formatNumber(avg_age,1,' ','.','','','','')+'</td><td style="width: 8%">&nbsp;</td>'+
	  '<td style="width: 28%;"><b>'+fsok_language.avg_form+':</b></td>'+
	  '<td style="text-align: right; width=18%">'+formatNumber(avg_form,1,' ','.','','','','')+'</td></tr>'+
	  '<tr><td colspan="5"><div class="separatorLine" style="width: 100%"></div></td></tr>'+
	  '<tr><td style="width: 28%;"><b>'+fsok_language.foreign_players+':</b></td>'+
	  '<td style="text-align: right; width=18%">'+statistics.strangers+'</td><td style="width: 8%">&nbsp;</td>';

	tab_content += '<td style="text-align:center;" colspan="2">'+
	  ((statistics.yellow_cards_1 > 0) ? '<img src="'+img_ycard+'" border="0" > ('+statistics.yellow_cards_1+')&nbsp;' : '')+
	  ((statistics.yellow_cards_2 > 0) ? '<img src="'+img_ycard+'" border="0" ><img src="'+img_ycard+'" border="0" > ('+statistics.yellow_cards_2+')&nbsp;' : '')+
	  ((statistics.red_cards > 0) ? '<img src="'+img_rcard+'" border="0" > ('+statistics.red_cards+')&nbsp;' : '')+
	  ((statistics.injuries > 0) ? '<img src="'+img_injury+'" border="0" > ('+statistics.injuries+statistics.injury_separator+statistics.injurydays+' '+fsok_language.days+')' : '')+
	  '</td></tr>';
	tab_content += '</tbody></table>';
	new_div_2_1_b_2.innerHTML = tab_content;
	tab.appendChild(new_div_1);
	tab.appendChild(new_div_2);
	tab.appendChild(new_div_3);

	var divs = doc.getElementsByTagName('div');
	for (var i = 0; i < divs.length; i++) {
	  if (divs[i].className == 'mainContainer space1Top') {
		divs[i].insertBefore(tab,divs[i].firstChild);
		break;
	  }
	}
  }

  /* Function: fsok_changeSortingOnPage
   *
   * This function actually changes the real page. It is called
   * by the event listeners defined above.
   *
   * @return  void
   */
  function fsok_changeSortingOnPage() {
	var updown = this.parentNode.getElementsByTagName('img')[0];
	var sorting = this.parentNode.getElementsByTagName('select')[0];
	if (this.getAttribute('id') == 'fsok_asc_arrow') {
	  if (updown.src == 'chrome://firesokker/content/icons/up_arrow.png')
		updown.src = 'chrome://firesokker/content/icons/down_arrow.png';
	  else
		updown.src = 'chrome://firesokker/content/icons/up_arrow.png';
	}
	var desc = (updown.src == 'chrome://firesokker/content/icons/down_arrow.png');

	try {
	  fsok_sortPlayers(sorting.value,desc);
	  var sortdiv = fsok_players_document.getElementById('sortdiv');
	  var l = player_divs_to_sort.length;
	  for (var i = 0; i < l; i++) {
		sortdiv.appendChild(player_divs_to_sort[i]);
		sortdiv.appendChild(clear_divs[i]);
	  }
	} catch (e) {
	  fsok_log(e, fsok_players_document);
	}
  }
}

/* Function: fsok_sortPlayers
 *
 * This function sorts the players. The sorting method is mergesort.
 * Credits for the implementation go to: http://w3.ift.ulaval.ca/~phcho8/mergesort/
 *
 * @param string   sort_type   the sorting type
 * @param bool     desc         sort in ascending or descending order
 *
 * @return void
 */
function fsok_sortPlayers(sort_type,desc) {

  var new_array = mergesort(player_divs_to_sort);
  player_divs_to_sort = new_array;

  // less than or equal operator for mergesort
  function lt_or_eq(obj1, obj2) {
	if (sort_type == 'value' || sort_type == 'age' || sort_type == 'salary') {
	  return (desc) ? parseInt(obj1.getAttribute(sort_type)) >= parseInt(obj2.getAttribute(sort_type)) :
		parseInt(obj1.getAttribute(sort_type)) <= parseInt(obj2.getAttribute(sort_type));
	} else {
	  return (desc) ? obj1.getAttribute(sort_type) >= obj2.getAttribute(sort_type) :
		obj1.getAttribute(sort_type) <= obj2.getAttribute(sort_type);
	}
  }

  // the main mergesort function
  function mergesort(arr) {

	var left = new Array();
	var right = new Array();
	var len = arr.length;

	if (len <= 1) return arr;

	var middle = Math.floor(len / 2);
	for (var i = 0; i < middle; i++) left.push(arr[i]);
	for (var i = middle; i < len; i++) right.push(arr[i]);

	left = mergesort(left);
	right = mergesort(right);

	return merge(left, right);
  }

  // merge two arrays
  function merge(left, right) {
	var retval = new Array();
	while(left.length > 0 && right.length > 0) {
	  if (lt_or_eq(left[0], right[0])) retval.push(left.shift());
	  else retval.push(right.shift());
	}

	if (left.length > 0) return retval.concat(left);
	if (right.length > 0) return retval.concat(right);

	return retval;
  }
}

if (!fsok_functions["/players.php"])
  fsok_functions["/players.php"] = new Array();

fsok_functions["/players.php"][fsok_functions["/players.php"].length] = fsok_makePlayersSortable;
